/* super-search
Author: Kushagra Gour (https://kushagragour.dev)
MIT Licensed

Mods in SS JS : BrightHaven  (https://brighthaven.net)
Mod Date      : 2019-02-24
Mod Name      : ssimp        (super-search implementation)
Mod Ver       : 1.0.0
Mod Purpose   : Provide Minimal Mistakes Theme with an RSS based search alternative
Mod Changelog : Starting with Kushagra's Super-Search script as a base for RSS scraping
                functionality the following has been changed/added
                - change queryselector names to bring in line with Minimal Mistakes
                - add handleinput function call to init
                - add currentinput check, and give " " value if first run
                - remove less than three input length check
                  (behavior change)
                  instead of blank page, display results of " " (result is all items shown)
                - change searchresults output html to bring in line with Minimal Mistakes
                - add description to searchresults output
                - remove date from searchresults output
                - remove '/' key event listener and assignment
                - change ESC key assignment function to handle the closing of search toggle
                - add site-page scroll-to-top function to ESC key assignment function
                - rename js file to bring in line with this Jekyll search plugin
                - add functions for searchbar toggle handling
                - add functions to properly handle hash url fragment links when already on page
*/
function scrollToTheTop() {
    // without animation
    //$('html,body').scrollTop(0);
    // with animation
    $('html, body').animate({ scrollTop: 0 }, 'fast');
}
function searchBar__toggle() {
    $(".search-content").toggleClass("is--visible");
    $(".initial-content").toggleClass("is--hidden");
}
function scrollToHashElement(ele) {
    $(window).scrollTop(ele.offset().top).scrollLeft(ele.offset().left);
}
function codeForHashLinkClicks() {
    // When clicking a link check if it has a hash url fragment
    // then let the browser know it can toggle the search bar away
    // and scroll to the hash element if it happens to be that the
    // destination page is the page the browser is already on
    // -- MM note: mainly when using search__form instead of search page
    $("a").click(function () {
        var searchContent = document.getElementById('search-content');
        var hash = this.hash;
        if (hash != "" || hash!= null) {
            if (searchContent.classList.contains('is--visible')) {
                searchBar__toggle();
                scrollToHashElement(hash);
            }             
        }
    });
}
// --MM note: added xtra functionality within original Jquery function (Search toggle with page scroll)
$(".searchbar__toggle").on("click", function() {
    searchBar__toggle();
    // set focus on input
    setTimeout(function() {
      $(".search-content input").focus();
    }, 400);
    scrollToTheTop();
    var searchContent = document.getElementById('search-content');
    if (searchContent.classList.contains('is--visible')) {
         codeForHashLinkClicks();
    } else {
         $("a").unbind('click');
    }
});

(function () {
	var searchFile = '/feed.xml',
		searchBar,
		searchInputEl,
		searchResultsEl,
		currentInputValue = '',
		lastSearchResultHash,
		posts = [];

	// Changes XML to JSON
	// Modified version from here: http://davidwalsh.name/convert-xml-json
	function xmlToJson(xml) {
		// Create the return object
		var obj = {};
		if (xml.nodeType == 3) { // text
			obj = xml.nodeValue;
		}

		// do children
		// If all text nodes inside, get concatenated text from them.
		var textNodes = [].slice.call(xml.childNodes).filter(function (node) { return node.nodeType === 3; });
		if (xml.hasChildNodes() && xml.childNodes.length === textNodes.length) {
			obj = [].slice.call(xml.childNodes).reduce(function (text, node) { return text + node.nodeValue; }, '');
		}
		else if (xml.hasChildNodes()) {
			for(var i = 0; i < xml.childNodes.length; i++) {
				var item = xml.childNodes.item(i);
				var nodeName = item.nodeName;
				if (typeof(obj[nodeName]) == "undefined") {
					obj[nodeName] = xmlToJson(item);
				} else {
					if (typeof(obj[nodeName].push) == "undefined") {
						var old = obj[nodeName];
						obj[nodeName] = [];
						obj[nodeName].push(old);
					}
					obj[nodeName].push(xmlToJson(item));
				}
			}
		}
		return obj;
	}

	function getPostsFromXml(xml) {
		var json = xmlToJson(xml);
		// Atom 1.0 format
		if (json.entry && json.entry instanceof Array) {
			return json.entry;
		}
		// Atom 2.0 format
		else {
			return json.channel.item;
		}
	}

	window.closeSearchBackToTop = function closeSearchBackToTop() {

      if (searchBar.classList.contains('is--visible')) {
         searchBar__toggle();
         $(".search-content input").blur();
      }
      scrollToTheTop();
	}

	function handleInput() {
		var currentResultHash;

		currentInputValue = (searchInputEl.value + '').toLowerCase();
        // first run check
        if (!currentInputValue) {
            currentInputValue = ' ';
        }
		searchResultsEl.style.offsetWidth;

		var matchingPosts = posts.filter(function (post) {
			// Search `description` and `content` both to support 1.0 and 2.0 formats.
			if ((post.title + '').toLowerCase().indexOf(currentInputValue) !== -1 || ((post.description || post.content) + '').toLowerCase().indexOf(currentInputValue) !== -1) {
				return true;
			}
		});
		if (!matchingPosts.length) {
			searchResultsEl.classList.add('is-hidden');
		}
		currentResultHash = matchingPosts.reduce(function(hash, post) { return post.title + hash; }, '');
		if (matchingPosts.length && currentResultHash !== lastSearchResultHash) {
			searchResultsEl.classList.remove('is-hidden');
			searchResultsEl.innerHTML = matchingPosts.map(function (post) {
				return '<li><h2><a href="' + post.link + '">' + post.title + '</a></h2><p>' + post.description + '</p></li>';
			}).join('');
		}
		lastSearchResultHash = currentResultHash;
	}

	function init(options) {
		searchFile = options.searchFile || searchFile;
		searchInputEl = document.querySelector(options.inputSelector || '#search');
		searchResultsEl = document.querySelector(options.resultsSelector || '#result__items');
		searchBar = document.querySelector(options.searchSelector || '#search-content');

		var xmlhttp=new XMLHttpRequest();
		xmlhttp.open('GET', searchFile);
		xmlhttp.onreadystatechange = function () {
			if (xmlhttp.readyState != 4) return;
			if (xmlhttp.status != 200 && xmlhttp.status != 304) { return; }
			var node = (new DOMParser).parseFromString(xmlhttp.responseText, 'text/xml');
			node = node.children[0];
			posts = getPostsFromXml(node);
		}
		xmlhttp.send();

        $(document).ready(function(){
            // First run to populate page
			handleInput();
        });

		// ESC key actions
		window.addEventListener('keyup', function onKeyPress(e) {
			if (e.which === 27) {
				closeSearchBackToTop();
			}
		});

		searchInputEl.addEventListener('input', function onInputChange() {
			handleInput();
            codeForHashLinkClicks();
		});
	}

	init.keyESC = closeSearchBackToTop;

	window.ssimp = init;

})();
